Imports System.ComponentModel
Imports System.Runtime.InteropServices

' Because this is used inside a recursive routine, I wrote it as "Shared"
Class TranslateSID

#Region "API Region"
    'BOOL LookupAccountSid(
    '  LPCTSTR lpSystemName,
    '  PSID lpSid,
    '  LPTSTR lpName,
    '  LPDWORD cchName,
    '  LPTSTR lpReferencedDomainName,
    '  LPDWORD cchReferencedDomainName,
    '  PSID_NAME_USE peUse
    ');
    Private Declare Auto Function LookupAccountSid Lib "advapi32.dll" ( _
        ByVal lpSystemName As String, _
        ByVal lpSid As Byte(), _
        ByVal lpName As String, _
        ByRef cchName As Integer, _
        ByVal lpReferenceDomainName As String, _
        ByRef cchReferencedDomainName As Integer, _
        ByRef peUse As Integer _
    ) As Boolean

    Private Const NAME_SIZE As Integer = 128
#End Region

    ' We keep a cache of resolved sids.  This is faster than continously
    ' looking up the same sid over and over again.
    Public Shared SID_Cache As New Hashtable

    ' Translate a SID into an account name.  This is similar to the internal
    ' translate() method, but instead of throwing an exception upon a mapping
    ' error, this routine returns the SID as as string.
    Public Shared Function TranslateSidToName(ByVal Server As String, ByVal Sid As System.Security.Principal.SecurityIdentifier) As String
        Dim MachineName, NameString, name, domain_name As String
        Dim name_len, domain_len, peUse, LastError As Integer
        Dim sid_buf() As Byte

        ' have we already resolved this SID before?
        If SID_Cache.ContainsKey(Sid.Value) Then
            Return SID_Cache(Sid.Value).ToString
        End If

        ' are we doing this remotely? If so, we need to resolve the SID remotely
        If String.Compare(Environment.MachineName, Server, True) <> 0 Then
            MachineName = Server
        Else
            MachineName = ""
        End If

        ' prep work for the API
        name_len = NAME_SIZE
        domain_len = NAME_SIZE
        name = Space(name_len)
        domain_name = Space(domain_len)

        ReDim sid_buf(Sid.BinaryLength - 1)
        Sid.GetBinaryForm(sid_buf, 0)

        ' look up the Account associated with that SID
        If LookupAccountSid(MachineName, sid_buf, name, name_len, domain_name, domain_len, peUse) = False Then
            LastError = Marshal.GetLastWin32Error()
            ' if this is the expected ERROR_NONE_MAPPED error
            ' where we can't resolve the SID?
            If LastError = 1332 Then
                domain_len = 0
                name = Sid.Value
                name_len = Len(name)
            Else
                Throw New Win32Exception(LastError)
            End If
        End If

        If domain_len > 0 Then
            NameString = Left(domain_name, domain_len) & "\" & Left(name, name_len)
        Else
            NameString = Left(name, name_len)
        End If

        ' add it to the hashtable
        SID_Cache.Add(Sid.Value, NameString)

        Return NameString
    End Function
End Class
