Imports System.IO
Imports System.Security.AccessControl
Imports System.Security.Principal

Public Class Traverse
    Private fm As fmMain
    Private first, owner, needs_conversion As Boolean
    Private dt As DataTable
    Private db As AccessDatabase
    Private server, RootName As String
    Private f_count, d_count As Integer

    Public Property MainForm() As fmMain
        Get
            Return fm
        End Get
        Set(ByVal value As fmMain)
            fm = value
        End Set
    End Property

    Public Property IsFirst() As Boolean
        Get
            Return first
        End Get
        Set(ByVal value As Boolean)
            first = value
        End Set
    End Property

    Public Property IncludeOwner() As Boolean
        Get
            Return owner
        End Get
        Set(ByVal value As Boolean)
            owner = value
        End Set
    End Property

    Public Property DataTable() As DataTable
        Get
            Return dt
        End Get
        Set(ByVal value As DataTable)
            dt = value
        End Set
    End Property

    Public Property AccessDatabase() As AccessDatabase
        Get
            Return db
        End Get
        Set(ByVal value As AccessDatabase)
            db = value
        End Set
    End Property

    Public Property ServerName() As String
        Get
            Return server
        End Get
        Set(ByVal value As String)
            server = value
        End Set
    End Property

    Public Property RecordRootName() As String
        Get
            Return RootName
        End Get
        Set(ByVal value As String)
            RootName = value
        End Set
    End Property

    Public ReadOnly Property FileCount() As Integer
        Get
            Return f_count
        End Get
    End Property

    Public ReadOnly Property DirectoryCount() As Integer
        Get
            Return d_count
        End Get
    End Property

    Public Sub Doit(ByVal StartingDir As String)
        Dim p_di, di, di_array() As DirectoryInfoEx
        Dim fi, fi_array() As FileInfoEx
        Dim dr As DataRow
        Dim sid As SecurityIdentifier

        ' dress it up a wee bit...
        If Not StartingDir.EndsWith("\") Then
            StartingDir = StartingDir & "\"
        End If

        ' skip the recycle bin and other directories we don't care about...
        For Each skip As String In My.Settings.SkipList
            If StartingDir.EndsWith(skip, StringComparison.CurrentCultureIgnoreCase) Then
                Exit Sub
            End If
        Next

        Try
            p_di = New DirectoryInfoEx(StartingDir)
        Catch
            ' silently ignore these errors
            Exit Sub
        End Try

        ' keep 'em entertained...
        fm.lbStatus.Text = StartingDir
        Application.DoEvents()

        ' get the starting directory info (first run only)
        If first Then
            Try
                ' do we need to convert a UNC path?
                needs_conversion = CBool(Not (StartingDir.StartsWith(RootName)))

                dr = dt.NewRow
                dr("Server") = server
                dr("Path") = FormatPath(StartingDir)
                dr("Name") = p_di.Name
                dr("Ext") = p_di.Extension.ToLower
                dr("FSize") = 0
                If owner Then
                    sid = CType(p_di.GetAccessControl.GetOwner(GetType(SecurityIdentifier)), SecurityIdentifier)
                    dr("Owner") = Left(TranslateSID.TranslateSidToName(server, sid), 128)
                End If
                dr("Attrib") = DoAttrib(p_di.Attributes)
                dr("Modified") = p_di.LastWriteTime
                dr("Created") = p_di.CreationTime
                dr("Accessed") = p_di.LastAccessTime
                dt.Rows.Add(dr)
                first = False
                d_count += 1
            Catch ex As Exception
                dr = dt.NewRow
                dr("Server") = server
                dr("Path") = FormatPath(StartingDir)
                dr("Name") = Left("Error: " & ex.Message, 260)
                dt.Rows.Add(dr)
                ' not fatal
            End Try
        End If

        ' process files first
        Try
            fi_array = p_di.GetFiles
        Catch ex As Exception
            dr = dt.NewRow
            dr("Server") = server
            dr("Path") = FormatPath(StartingDir)
            dr("Name") = Left("Error: " & ex.Message, 260)
            dt.Rows.Add(dr)
            Exit Sub
        End Try

        For Each fi In fi_array
            Try
                dr = dt.NewRow
                dr("Server") = server
                dr("Path") = FormatPath(fi.FullName)
                dr("Name") = fi.Name
                dr("Ext") = fi.Extension.ToLower
                dr("FSize") = fi.Length
                If owner Then
                    sid = CType(fi.GetAccessControl.GetOwner(GetType(SecurityIdentifier)), SecurityIdentifier)
                    dr("Owner") = Left(TranslateSID.TranslateSidToName(server, sid), 128)
                End If
                dr("Attrib") = DoAttrib(fi.Attributes)
                dr("Modified") = fi.LastWriteTime
                dr("Created") = fi.CreationTime
                dr("Accessed") = fi.LastAccessTime
                dt.Rows.Add(dr)
                f_count += 1
            Catch ex As Exception
                dr = dt.NewRow
                dr("Server") = server
                dr("Path") = FormatPath(StartingDir)
                dr("Name") = Left("Error: " & ex.Message, 260)
                dt.Rows.Add(dr)
                ' not fatal
            End Try
        Next

        ' Let's keep the memory footprint down... 
        If dt.Rows.Count > My.Settings.RowBuffer Then
            fm.lbStatus.Text &= " (saving data)"
            Application.DoEvents()
            db.UpdateTable(dt)
            dt.Clear()
        End If

        ' Process the directories (and descend into dir structure)
        Try
            di_array = p_di.GetDirectories
        Catch ex As Exception
            ' This probably redundant, since we catch this type of
            ' error above while processing files.
            dr = dt.NewRow
            dr("Server") = server
            dr("Path") = FormatPath(StartingDir)
            dr("Name") = Left("Error: " & ex.Message, 260)
            dt.Rows.Add(dr)
            Exit Sub
        End Try

        For Each di In di_array
            Try
                dr = dt.NewRow
                dr("Server") = server
                dr("Path") = FormatPath(di.FullName)
                dr("Name") = di.Name
                dr("Ext") = di.Extension.ToLower
                dr("FSize") = 0
                If owner Then
                    sid = CType(di.GetAccessControl.GetOwner(GetType(SecurityIdentifier)), SecurityIdentifier)
                    dr("Owner") = Left(TranslateSID.TranslateSidToName(server, sid), 128)
                End If
                dr("Attrib") = DoAttrib(di.Attributes)
                dr("Modified") = di.LastWriteTime
                dr("Created") = di.CreationTime
                dr("Accessed") = di.LastAccessTime
                dt.Rows.Add(dr)
                d_count += 1
            Catch ex As Exception
                dr = dt.NewRow
                dr("Server") = server
                dr("Path") = FormatPath(StartingDir)
                dr("Name") = Left("Error: " & ex.Message, 260)
                dt.Rows.Add(dr)
                ' not fatal
            End Try

            ' recursive call to this subroutine
            Doit(di.FullName)
        Next
    End Sub

    '
    ' Format the Path field.  Converts a UNC path back to it's drive letter
    ' equivalent.
    '
    Private Function FormatPath(ByVal Path As String) As String
        If needs_conversion Then
            Static index As Integer = -1
            Static DriveLetter As String

            If index < 0 Then
                ' these values don't change, so we can make 'em static
                index = Path.IndexOf("$")
                DriveLetter = Path.Substring(index - 1, 1).ToUpper & ":"
            End If

            Return Left(DriveLetter & Path.Substring(index + 1), 4000)
        Else
            Return Left(Path, 4000)
        End If
    End Function

    '
    ' Convert the attributes to the one-letter equivalents
    '
    Private Function DoAttrib(ByVal attr As FileAttributes) As String
        Dim sb As New System.Text.StringBuilder

        If CInt(attr) = -1 Then
            Return "Error"
        End If

        If (attr And FileAttributes.Archive) <> 0 Then
            sb.Append("a")
        End If
        If (attr And FileAttributes.Compressed) <> 0 Then
            sb.Append("c")
        End If
        If (attr And FileAttributes.Device) <> 0 Then
            sb.Append("D")
        End If
        If (attr And FileAttributes.Directory) <> 0 Then
            sb.Append("d")
        End If
        If (attr And FileAttributes.Encrypted) <> 0 Then
            sb.Append("e")
        End If
        If (attr And FileAttributes.Hidden) <> 0 Then
            sb.Append("h")
        End If
        If (attr And FileAttributes.Normal) <> 0 Then
            sb.Append("n")
        End If
        If (attr And FileAttributes.NotContentIndexed) <> 0 Then
            sb.Append("I")
        End If
        If (attr And FileAttributes.Offline) <> 0 Then
            sb.Append("O")
        End If
        If (attr And FileAttributes.ReadOnly) <> 0 Then
            sb.Append("r")
        End If
        If (attr And FileAttributes.ReparsePoint) <> 0 Then
            sb.Append("P")
        End If
        If (attr And FileAttributes.SparseFile) <> 0 Then
            sb.Append("S")
        End If
        If (attr And FileAttributes.System) <> 0 Then
            sb.Append("s")
        End If
        If (attr And FileAttributes.Temporary) <> 0 Then
            sb.Append("T")
        End If

        Return sb.ToString
    End Function
End Class
